"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const debug_1 = __importDefault(require("debug"));
const types_1 = require("./types");
const interface_1 = require("./interface");
class Logger {
    constructor(prefix) {
        const scopedPrefix = `framer:${prefix}`;
        const infoDebug = debug_1.default(scopedPrefix);
        infoDebug.log = console.log.bind(console);
        const errorDebug = debug_1.default(scopedPrefix);
        this.infoLogger = this.wrap(infoDebug);
        this.errorLogger = this.wrap(errorDebug);
    }
    static configure({ output = types_1.LogOutput.debug }) {
        Logger.config.output = output;
    }
    [types_1.LogTypes.debug](message, ...additional) {
        return this.infoLogger(message, ...additional);
    }
    [types_1.LogTypes.info](message, ...additional) {
        return this.infoLogger(message, ...additional);
    }
    [types_1.LogTypes.log](message, ...additional) {
        return this.infoLogger(message, ...additional);
    }
    [types_1.LogTypes.warn](message, ...additional) {
        return this.errorLogger(message, ...additional);
    }
    [types_1.LogTypes.error](message, ...optionalParams) {
        return this.errorLogger(message, ...optionalParams);
    }
    wrap(printer) {
        return (...args) => {
            if (!printer.enabled || Logger.config.output !== types_1.LogOutput.debug) {
                return;
            }
            if (interface_1.spinner.isSpinning) {
                interface_1.spinner.stop();
                printer(...args);
                interface_1.spinner.start();
            }
            else {
                printer(...args);
            }
        };
    }
}
exports.Logger = Logger;
Logger.config = {
    output: types_1.LogOutput.debug,
};
//# sourceMappingURL=Logger.js.map