"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.publish = void 0;
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const build_library_1 = require("@framerjs/build-library");
const config_1 = require("../config");
const interface_1 = require("../interface");
const validators = __importStar(require("../validation"));
const utils_1 = require("../utils");
const Api_1 = require("../Api");
const validation_1 = require("../validation");
exports.publish = (argv) => __awaiter(void 0, void 0, void 0, function* () {
    // Validate auth, package, and argv
    interface_1.spinner.start("Validating");
    // Need FRAMER_TOKEN to setup .npmrc file correctly
    // which is needed to access packages from the Framer store
    // and publish the package
    validators.framerTokenIsSet();
    // Build the configuration needed for publishing
    const config = config_1.buildPublishConfig(argv);
    const api = new Api_1.API(config.api);
    // Check authentication
    const userInfo = yield api.getUserInfo();
    validation_1.teamIdIsSpecified(config.isNewArgSet, config.spaceId, config.isPrivate, userInfo.spaces);
    // Get the .npmrc template
    config.project.npmrcTemplate = userInfo.npmrc;
    const project = yield build_library_1.Project.load(config.project);
    // Ensure there is sufficient data for checking the registry
    // (package name, display name, or new argument input)
    yield validators.canFetchPackageMetaData(project, config.newName);
    // Check that the user can publish to the requested store
    validators.canPublishToStore(config.isPrivate, userInfo);
    let depsMeta;
    interface_1.spinner.text = "Checking dependencies";
    const deps = yield project.getDependencies();
    const depsArr = Object.keys(deps.dependencies);
    if (depsArr.length) {
        depsMeta = yield api.getPackagesMeta(depsArr);
    }
    let packageName = yield project.getPackageName();
    // Get package data from registry about the package
    const { version: latestPublishedVersion, packageName: remotePackageName, displayName, published, spaceId, } = yield api.getPackageMeta(packageName, (yield project.getDisplayName()) || config.newName, config.isPrivate, config.spaceId);
    if (!packageName) {
        packageName = remotePackageName;
    }
    const targetSpaceId = spaceId || config.spaceId || undefined;
    if (published && config.spaceId) {
        // Package can not change it's team once published
        validators.spaceIsNotNew(userInfo, config.spaceId, spaceId);
    }
    if (depsMeta) {
        // If there are any deps — checking that they match current package team
        validators.hasNoIssuesWithTeamsPermissions(depsMeta, config.isPrivate, targetSpaceId);
    }
    // Check that the user can publish to the requested store
    validators.canPublishToSpace(config.isPrivate, userInfo, targetSpaceId);
    // Prevent users from publishing with the `--new` argument
    // if the package is already in the registry
    validators.newPackageCanBePublished(published, config.isNewArgSet, displayName, packageName);
    interface_1.spinner.text = "Preparing";
    // Use remote data and local package to pick the next version to publish
    const nextVersion = getNextVersion(yield project.getPackageVersion(), latestPublishedVersion, argv.major !== undefined ? "major" : "minor", published);
    interface_1.spinner.text = "Building project";
    const compiler = yield project.buildDist();
    if (compiler.dependencies.missing.length > 0) {
        interface_1.printMissingDependenciesWarning(compiler.dependencies.missing, project.path);
    }
    // Confirm the user wants to publish the version
    // (unless the --yes flag is passed in)
    if (!config.disablePrompts) {
        interface_1.spinner.stop();
        yield promptBeforePublishing(packageName, nextVersion, config.isPublic, userInfo, targetSpaceId);
        interface_1.spinner.start();
    }
    interface_1.spinner.text = "Publishing";
    // Prepare the project for preflight, then publish, and finally post-processing
    yield api.preflight(nextVersion, packageName, config.isPrivate, targetSpaceId);
    // If there are any errors, report and reject the pending version
    const result = yield project.publish({
        version: nextVersion,
        packageName,
        displayName,
        access: config.isPublic ? "public" : "restricted",
    });
    if (result.error.length) {
        const errors = result.error.map(({ data }) => data);
        // We don't care if the reject call fails (for the user at least)
        yield api.rejectPendingVersion(packageName, nextVersion, errors.join("\n")).catch();
        throw new Error(chalk_1.default `Something went wrong publishing {bold ${packageName}@${nextVersion}}:\n${errors.join("\n")}.\nPlease re-run the {bold {underline publish}} command.`);
    }
    // Poll on the registry check to account for post processing
    yield checkPackageVersionStatus(api, packageName, nextVersion);
    interface_1.spinner.stopAndPersist({
        text: "Package successfully published",
        symbol: "🚀",
    });
});
function getNextVersion(pkgVersion = "", latestPublishedVersion = "", bump, previouslyPublished) {
    // 1) If the package has not been previously published, set the
    //    version to 1.0.0, regardless of the package version
    // 2) If there is a  package version and a latest published version,
    //    and the package version is higher, take the package version
    //    without bumping
    // 3) If there is a package version and a latest published version,
    //    and the package version is equal to or less than the latest
    //    published version, bump the latest published version
    //    based on versioning strategy
    // 4) If there is a package version and no latest published version,
    //    take the package version as is
    // 5) If there is no package version and a latest published version,
    //    bump the latest published version based on versioning strategy
    // 6) Otherwise, default to version 1.0.0
    const defaultVersion = "1.0.0";
    const hasValidPackageVersion = !!semver_1.default.valid(pkgVersion);
    const hasValidLatestPublishedVersion = !!semver_1.default.valid(latestPublishedVersion);
    // 1
    if (!previouslyPublished) {
        return defaultVersion;
    }
    // 2
    if (hasValidPackageVersion &&
        hasValidLatestPublishedVersion &&
        semver_1.default.gt(pkgVersion || "", latestPublishedVersion || "")) {
        return pkgVersion;
    }
    // 3
    if (hasValidPackageVersion && hasValidLatestPublishedVersion && semver_1.default.lte(pkgVersion, latestPublishedVersion)) {
        return semver_1.default.inc(latestPublishedVersion, bump);
    }
    // 4
    if (hasValidPackageVersion && !hasValidLatestPublishedVersion) {
        return pkgVersion;
    }
    // 5
    if (!hasValidPackageVersion && hasValidLatestPublishedVersion) {
        return semver_1.default.inc(latestPublishedVersion, bump);
    }
    // 6
    return defaultVersion;
}
function promptBeforePublishing(name, version, isPublic, userInfo, spaceId) {
    return __awaiter(this, void 0, void 0, function* () {
        let promptText = `Ready to publish `;
        promptText += chalk_1.default `{bold ${name}} v{bold ${version}} `;
        if (isPublic) {
            promptText += chalk_1.default `({yellowBright public store}). `;
        }
        else if (spaceId && userInfo.spaces) {
            const targetSpace = userInfo.spaces.find(s => s.id === spaceId);
            if (targetSpace) {
                promptText += chalk_1.default `({greenBright private store, team "${targetSpace.displayName}"}). `;
            }
        }
        else {
            promptText += chalk_1.default `({greenBright private store}). `;
        }
        promptText += `Continue (y/N)? `;
        const answer = yield interface_1.prompt(promptText);
        if (!answer.trim().match(/^(y|yes)$/i)) {
            interface_1.spinner.stopAndPersist({ text: "Publishing aborted", symbol: "🚫" });
            // Terminate the process cleanly
            process.exit();
        }
    });
}
function checkPackageVersionStatus(api, packageName, version) {
    return __awaiter(this, void 0, void 0, function* () {
        const genericErrorMessage = "Something went wrong publishing the package. Please try again.";
        let versionResponse;
        try {
            versionResponse = yield utils_1.poll(() => api.getPackageVersionStatus(packageName, version), (response) => response.status === "completed" || response.status === "rejected", 1000 * 2, 1000 * 60 * 3);
        }
        catch (error) {
            throw new Error(genericErrorMessage);
        }
        if (versionResponse.status === "rejected") {
            if (versionResponse.error.internal) {
                throw new Error(genericErrorMessage);
            }
            throw new Error(versionResponse.error.message);
        }
    });
}
//# sourceMappingURL=publish.js.map