"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkIfOutOfDate = void 0;
// Make sure that the CLI package is the latest version
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const pkg_dir_1 = __importDefault(require("pkg-dir"));
const latest_version_1 = __importDefault(require("latest-version"));
const configs = __importStar(require("./config"));
const interface_1 = require("./interface");
const Logger_1 = require("./Logger");
const logger = new Logger_1.Logger("update");
// Throw if current package is a full major version behind, otherwise warn
// Default wait time between version checks is 24 hours
function checkIfOutOfDate(updateCheckInterval = 24 * 60 * 60 * 1000) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = configs.loadLocalConfig();
        if (Date.now() - config.get("lastUpdateCheck") < updateCheckInterval) {
            logger.info("Package version checked recently.");
            return;
        }
        else {
            logger.info("Package version not checked recently.");
        }
        interface_1.spinner.start("Checking for updates");
        const pkg = yield pkg_dir_1.default(__dirname);
        if (!pkg) {
            throw new Error("Could not find the root directory of the Framer CLI package.\nPlease try re-installing the package or re-running the command.");
        }
        const pkgPath = path_1.default.join(pkg, "package.json");
        const { version: currentVersion, name } = JSON.parse(yield fs_extra_1.default.readFile(pkgPath, "utf8"));
        // Try to fetch the latest version from the registry
        // If the registry is down, silently move on
        let latestVersion;
        try {
            latestVersion = yield latest_version_1.default(name);
            if (!semver_1.default.valid(latestVersion)) {
                throw new Error("Invalid version returned from registry.");
            }
            config.set("lastUpdateCheck", Date.now());
            logger.info(`Latest version v${latestVersion}`);
        }
        catch (error) {
            logger.error(error);
            interface_1.spinner.stop();
            return;
        }
        const msg = chalk_1.default `Framer CLI is out of date.\nPlease update to {bold ${name}@${latestVersion}}.`;
        // On major version diffs, throw an error
        if (semver_1.default.diff(currentVersion, latestVersion) === "major") {
            throw new Error(msg);
        }
        if (semver_1.default.lt(currentVersion, latestVersion)) {
            interface_1.printWarning(msg);
        }
        interface_1.spinner.stop();
    });
}
exports.checkIfOutOfDate = checkIfOutOfDate;
//# sourceMappingURL=update.js.map