"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.newPackageCanBePublished = exports.hasNoIssuesWithTeamsPermissions = exports.canFetchPackageVersion = exports.canFetchPackageMetaData = exports.canPublishToSpace = exports.spaceIsNotNew = exports.canPublishToStore = exports.teamIdIsSpecified = exports.framerTokenIsSet = void 0;
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
exports.framerTokenIsSet = () => {
    const { FRAMER_TOKEN: token } = process.env;
    assert_1.default.ok(token && token.trim(), chalk_1.default `Missing {bold FRAMER_TOKEN} environment variable.\nPlease run the {bold {underline authenticate}} command to obtain your {bold FRAMER_TOKEN}.`);
};
exports.teamIdIsSpecified = (isNewArgSet, spaceId, storeIsPrivate, spaces) => {
    const writableSpaces = spaces && spaces.filter(s => s.permissions.write);
    if (writableSpaces && writableSpaces.length > 1) {
        if (storeIsPrivate && isNewArgSet && !spaceId) {
            throw new Error(`You must specify team id.\n\n${chalk_1.default `{bold {whiteBright Please select from the following list:}}`}\n\n${writableSpaces
                .map(space => chalk_1.default `{whiteBright ${space.id}}{grey  - ${space.displayName}}`)
                .join("\n")}`);
        }
    }
};
exports.canPublishToStore = (storeIsPrivate, userInfo) => {
    assert_1.default.ok(!storeIsPrivate || (userInfo.organization && userInfo.organization.id !== "00000000-0000-0000-0000-000000000000"), chalk_1.default `Package cannot be published privately.\nPlease re-run this command with the {bold --public} flag.`);
};
exports.spaceIsNotNew = (userInfo, targetSpaceId, publishedSpaceId) => {
    if (publishedSpaceId) {
        const space = userInfo.spaces && userInfo.spaces.find(s => s.id === publishedSpaceId);
        assert_1.default.ok(targetSpaceId === publishedSpaceId, chalk_1.default `Package can not change its team. You can make a copy and publish it as a new package to a different team${space ? `\nPackage belongs to "${space.displayName}" with id "${space.id}"` : ""}
      `);
    }
};
exports.canPublishToSpace = (storeIsPrivate, userInfo, spaceId) => {
    if (storeIsPrivate && spaceId && userInfo.spaces) {
        const targetSpace = userInfo.spaces.find(s => s.id === spaceId);
        assert_1.default.ok(targetSpace && targetSpace.permissions.write, 
        // TODO remove available spaces when add new command
        chalk_1.default `Team ${spaceId} is not available for publishing. To get list of available teams run with {bold teams-list} command`);
    }
};
exports.canFetchPackageMetaData = (project, newName) => __awaiter(void 0, void 0, void 0, function* () {
    assert_1.default.ok((yield project.getPackageName()) || (yield project.getDisplayName()) || newName, chalk_1.default `Cannot fetch package metadata because of missing information.\nPlease re-run this command with a {bold display name} using the {bold --new} argument.`);
});
exports.canFetchPackageVersion = (project) => __awaiter(void 0, void 0, void 0, function* () {
    assert_1.default.ok(yield project.getPackageName(), chalk_1.default `Cannot fetch the latest version because of missing information.\nPlease ensure the package was previously published.`);
});
exports.hasNoIssuesWithTeamsPermissions = (deps, isPrivate, targetSpaceId) => {
    const depsArray = Object.values(deps);
    const uniqueSpaces = Array.from(depsArray
        .reduce((base, dep) => {
        if (dep.spaceId)
            base.add(dep.spaceId);
        return base;
    }, new Set())
        .values());
    const hasPrivateDeps = depsArray.some(dep => dep && dep.isPrivate);
    if (!isPrivate) {
        assert_1.default.ok(!hasPrivateDeps, chalk_1.default `Projects with team dependencies cannot be published publicly`);
    }
    assert_1.default.ok(uniqueSpaces.length <= 1, chalk_1.default `Projects has dependencies on packages from various teams`);
    const hasDepsOnDifferentTeams = isPrivate && targetSpaceId && uniqueSpaces.length && targetSpaceId !== uniqueSpaces[0];
    assert_1.default.ok(!hasDepsOnDifferentTeams, chalk_1.default `Projects has dependencies on packages from other than selected team`);
};
exports.newPackageCanBePublished = (published, isNewArgSet, displayName, packageName) => {
    assert_1.default.ok(!(published && isNewArgSet), chalk_1.default `This package has already been published as {bold ${displayName}} (${packageName}).
Please re-run the publish command without the \`--new\` argument.`);
};
//# sourceMappingURL=validation.js.map